package yu_tcp

import (
	"fmt"
	yu_proxy "gogs.qqck.cn/s/tools/proxy"
	"testing"
)

func Test(t *testing.T) {
	j_tcp := New()
	defer j_tcp.Close()
	j_tcp.SetProxy(yu_proxy.NewInfo().SetHost("127.0.0.1").SetPort(808).SetHttp().SetTimeout(-1))
	// fmt.Println(j_tcp.SetHost("fe80::fd70:67ee:a6a2:52e%18").SetPort(65522).Connect())
	// fmt.Println(j_tcp.SetHost("127.0.0.1").SetPort(65522).Connect())
	fmt.Println(j_tcp.SetHost("www.baidu.com").SetPort(80).Connect())

}
